/////////////////////////////////////////////////////////
//
//Name:			buffer	
//
//Author:		David McKelvie daveatsubversedotcodotnz
//
//Description:	Circular buffer
//
//Targets:		68HC11/PIC/AVR
//
//Version:		
//
//Functions:	bufferInitialise()
//				bufferPut()
//				bufferGet()
//
//Usage:		Declare a variable of type "strBuffer",
//				
//				call bufferInitialise() with a pointer 
//				to the strBuffer, then call:
//					bufferPut()
//					bufferGet()
//				which return TRUE if successfull or
//				FALSE on failure.	
//
//	Copyright (C) 2005 David McKelvie
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
/////////////////////////////////////////////////////////
#include<stdint.h>
#include"util.h"
#include"buffer.h"

//////////////////////////////////////////////////////////
//
//Name:			bufferInitialise
//
//////////////////////////////////////////////////////////
uint8_t bufferInitialise( strBuffer *ptrBuffer )
{
	ptrBuffer -> head = 0;			//set indexes
	ptrBuffer -> tail = 0;
	return TRUE;
}

//////////////////////////////////////////////////////////
//
//Name:			bufferGet
//
//Description:	takes one character from buffer, placing
//				it at the location pointed to by ptrChar
//
//////////////////////////////////////////////////////////
uint8_t bufferGet( uint8_t *ptrChar, strBuffer *ptrBuffer )
{
	if( ptrBuffer -> head == ptrBuffer -> tail )			//is buffer empty
		return FALSE;
	*ptrChar = ptrBuffer -> buffer[ ptrBuffer -> head ++ ];	//get char
	if( ptrBuffer -> head == BSIZE )						//circulate buffer
		ptrBuffer -> head = 0;
	return TRUE;
}

//////////////////////////////////////////////////////////
//
//Name:			bufferPut
//
//Description:	places data in buffer
//
//////////////////////////////////////////////////////////
uint8_t bufferPut( uint8_t data, strBuffer *ptrBuffer )
{
	if( ptrBuffer -> head == ( ( ptrBuffer -> tail + 1 ) % BSIZE ) )		//is buffer full
		return FALSE;
	ptrBuffer -> buffer[ ptrBuffer -> tail ] = data;		//put char
	if( ++ ptrBuffer -> tail == BSIZE )					//circulate  buffer
		ptrBuffer -> tail = 0;
	return TRUE;
}
