/*
 * T030 ROM BIOS - EhBasic support routines
 *
 * (c) 2016 Tobias Rathje <tobbe@fasanix.com>
 *
 */

#include <stdint.h>
#include "vector.h"
#include "mfp.h"
#include "misc.h"
#include "sd.h"

#define EHBASIC_ADDR 0xc0196

/*
 * EhBasic IO handler
 */
void trap15_handler()
{
	static uint8_t op, arg;

	__asm("move.b %%d0,%0":"=d"(op));
	__asm("move.b %%d1,%0":"=d"(arg));

	switch (op)
	{
		case 5:
			__asm("move.b %0,%%d1":: "d"(MFPUDR));
			break;
		case 6:
			mfp_putc(arg);
			break;
		case 7:
			if(bit_is_clear(MFPRSR, 7))
				__asm("move.b %0,%%d1":: "d"(0));
			else
				__asm("move.b %0,%%d1":: "d"(MFPUDR));
			break;
		default:
			break;
	}

	__asm("rte");
}

/*
 * Run EhBasic
 */
void run_basic()
{
	static uint32_t addr = EHBASIC_ADDR;

	// load EhBasic binary from sd card
	sd_cmd("r ehbasic.bin c0000");

	// install trap 15 handler
	set_vector(47, trap15_handler);

	// enable cache
	cache_ctl(0x101);

	// switch to user mode / no interrupts
	__asm("move.w #0x0700, %sr");

	// set up user stack
	__asm("move.l #0x000efffc, %sp");

	// GO!
	__asm("move.l %0,%%a0":: "d"(addr));
	__asm("jsr @(%a0)");
}
