/*
 * T030 ROM BIOS - Main routine
 *
 * (c) 2016 Tobias Rathje <tobbe@fasanix.com>
 *
 */

#include <stdio.h>
#include <stdint.h>
#include "vector.h"
#include "mfp.h"
#include "command.h"
#include "misc.h"
#include "sd.h"
#include "script.h"

int main(void)
{
    // initialize exception vectors
    init_vector();

    // initialize MFP
    mfp_init();

    // print banner
    mfp_puts("\r\nT030 ROM BIOS v1.52 (c) 2016 Tobias Rathje\r\n");
    mfp_puts("------------------------------------------\r\n\r\n");

    mfp_printf("Build: %s %s\r\n", __DATE__, __TIME__);

    // initalize SD module
    sd_init();

    // enable interrupts
    sei();

    // enable cache
    cache_ctl(0x101);

    // yellow led off, green on
    bit_set(MFPGPDR, 0);
    bit_clear(MFPGPDR, 1);

    // run init script if found
    run_init_script();

    // command interpreter
    cmd_loop();
}
