/*
 * T030 ROM BIOS - TFTP loader
 *
 * (c) 2016 Tobias Rathje <tobbe@fasanix.com>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mfp.h"
#include "misc.h"

#include "ip_arp_udp_tcp.h"
#include "enc28j60.h"
#include "net.h"
#include "dhcp_client.h"

#include "tftp.h"

#define BUFFER_SIZE 500
static uint8_t buf[BUFFER_SIZE + 1];

static uint8_t mymac[6] = { 0xcd, 0x15, 0x54, 0x30, 0x33, 0x30 };

static uint8_t myip[4] = { 0, 0, 0, 0 };
static uint8_t gwip[4] = { 0, 0, 0, 0 };
static uint8_t dip[4] = { 0, 0, 0, 0 };

#define TRANS_NUM_GWMAC 1
static uint8_t gwmac[6];
static uint8_t netmask[4] = { 255, 255, 255, 0 };


void arpresolver_result_callback(uint8_t *ip __attribute__((unused)), uint8_t reference_number, uint8_t *mac)
{
	uint8_t i = 0;
	if (reference_number == TRANS_NUM_GWMAC)
	{
		// copy mac address over:
		while (i < 6)
		{
			gwmac[i] = mac[i];
			i++;
		}
	}
}

int parse_ip_str(char *ip_str, uint8_t *ip)
{
	int a,b,c,d;

	int ret = siscanf(ip_str, "%d.%d.%d.%d", &a, &b, &c, &d);

	if (ret < 4)
	{
		mfp_printf("IP format error: %s\r\n", ip_str);
		return -1;
	}

	*ip++ = a;
	*ip++ = b;
	*ip++ = c;
	*ip = d;

	return 0;
}

void do_netload(char *lcl_ip_str, char *rmt_ip_str, char *fname, uint32_t addr)
{
	uint16_t plen;
	uint8_t rval;

	//initialize enc28j60
	enc28j60Init(mymac);

	mfp_delay_ms(100);

	// Magjack leds configuration, see enc28j60 datasheet, page 11
	// LEDB=yellow LEDA=green
	// 0x476 is PHLCON LEDA=links status, LEDB=receive/transmit
	// enc28j60PhyWrite(PHLCON,0b0000 0100 0111 01 10);
	enc28j60PhyWrite(PHLCON, 0x476);

	mfp_delay_ms(100);

	rval = 0;
	init_mac(mymac);

	if (strcmp(lcl_ip_str, "dhcp") == 0)
	{
		mfp_printf("Requesting DHCP address... ");
		while (rval == 0)
		{
			plen = enc28j60PacketReceive(BUFFER_SIZE, buf);
			buf[BUFFER_SIZE] = '\0';
			rval = packetloop_dhcp_initial_ip_assignment(buf, plen, mymac[5]);
		}

		// we have an IP:
		dhcp_get_my_ip(myip, netmask, gwip);

		mfp_printf("OK\r\n\r\n");
	}
	else
	{
		if (parse_ip_str(lcl_ip_str, myip) < 0)
			return;

		// assume gw ip
		gwip[0] = myip[0];
		gwip[1] = myip[1];
		gwip[2] = myip[2];
		gwip[3] = 1;
	}

	client_ifconfig(myip, netmask);

	mfp_printf("Local IP address: %d.%d.%d.%d\r\n\r\n", *myip, *(myip+1), *(myip+2), *(myip+3));

	get_mac_with_arp(gwip, TRANS_NUM_GWMAC, &arpresolver_result_callback);
	while (get_mac_with_arp_wait())
	{
		// to process the ARP reply we must call the packetloop
		plen = enc28j60PacketReceive(BUFFER_SIZE, buf);
		packetloop_arp_icmp_tcp(buf, plen);
	}

	if (parse_ip_str(rmt_ip_str, dip) < 0)
		return;

	mfp_printf("Server IP address: %d.%d.%d.%d - File: %s\r\n\r\n", *dip, *(dip+1), *(dip+2), *(dip+3), fname);

	recv_file(dip, gwmac, fname, addr);
}

