/*
 * T030 ROM BIOS - RAM test
 *
 * (c) 2016 Tobias Rathje <tobbe@fasanix.com>
 *
 */

#include <stdio.h>
#include "mfp.h"

void check_base_ram()
{
	int chk, i = 0;
	volatile uint8_t *p;
	uint8_t *r = (uint8_t *)0x80000;

	chk = 1;

	for (i = 10; i < 512; i++)
	{
		p = r + (i * 1024);

		mfp_printf("\rChecking RAM: %d KB ", i + 1);

		*p = 0x76;
		if (*p != 0x76)
			chk = 0;

		*p = 0xf2;
		if (*p != 0xf2)
			chk = 0;

		if (!chk)
		{
			mfp_printf("FAILED at %p\r\n", p);
			break;
		}
	}

	if (chk)
		mfp_printf("OK\r\n");
}

void check_ext_ram()
{
	int chk, i = 0;
	volatile uint8_t *p;
	uint8_t *r = (uint8_t *)0x1000000;

	chk = 1;

	for (i = 1; i < 16384; i++)
	{
		p = r + (i * 1024);

		mfp_printf("\rChecking extended RAM: %d KB ", i + 1);

		*p = 0x76;
		if (*p != 0x76)
			chk = 0;

		*p = 0xf2;
		if (*p != 0xf2)
			chk = 0;

		if (!chk)
		{
			mfp_printf("FAILED at %p\r\n", p);
			break;
		}
	}

	if (chk)
		mfp_printf("OK\r\n");
}

void check_ram_8(uint32_t addr, uint32_t size, uint8_t cnt)
{
	int chk = 1;
	volatile uint8_t *p;
	uint8_t *r = (uint8_t *)addr;
	uint16_t i;
	uint32_t j;
	// uint8_t k;

	for (i = 0; i < cnt; i++)
	{
		mfp_printf("Run %d\r\n", i);

		for (j = 0; j < size; j++)
		{
			p = r + j;

			*p = 0x55;
			if (*p != 0x55)
				chk = 0;

			*p = 0xaa;
			if (*p != 0xaa)
				chk = 0;

			/*
			for (k = 0; k < 0xff; k++)
			{
				*p = k;
				if (*p != k)
					chk = 0;
			}
			*/

			if (!chk)
			{
				mfp_printf("FAILED at %p\r\n", p);
				chk = 1;
				// break;
			}
		}
	}
}

void check_ram_16(uint32_t addr, uint32_t size, uint8_t cnt)
{
	int chk = 1;
	volatile uint16_t *p;
	uint16_t *r = (uint16_t *)addr;
	uint16_t i;
	uint32_t j;

	for (i = 0; i < cnt; i++)
	{
		mfp_printf("Run %d\r\n", i);

		for (j = 0; j < size / 2; j++)
		{
			p = r + j;

			*p = 0x5555;
			if (*p != 0x5555)
				chk = 0;

			*p = 0xaaaa;
			if (*p != 0xaaaa)
				chk = 0;

			if (!chk)
			{
				mfp_printf("FAILED at %p\r\n", p);
				chk = 1;
			}
		}
	}
}

void check_ram_32(uint32_t addr, uint32_t size, uint8_t cnt)
{
	int chk = 1;
	volatile uint32_t *p;
	uint32_t *r = (uint32_t *)addr;
	uint16_t i;
	uint32_t j;

	for (i = 0; i < cnt; i++)
	{
		mfp_printf("Run %d\r\n", i);

		for (j = 0; j < size / 4; j++)
		{
			p = r + j;

			*p = 0x55555555;
			if (*p != 0x55555555)
				chk = 0;

			*p = 0xaaaaaaaa;
			if (*p != 0xaaaaaaaa)
				chk = 0;

			if (!chk)
			{
				mfp_printf("FAILED at %p\r\n", p);
				chk = 1;
			}
		}
	}
}
