/*
 * T030 ROM BIOS - Script runner
 *
 * (c) 2016 Tobias Rathje <tobbe@fasanix.com>
 *
 */

#include <string.h>
#include "mfp.h"
#include "sd.h"
#include "command.h"

char sbuf[4096];
char lbuf[80];

/*
 * Run script line
 */
void run_line(char *line)
{
	if (*line == '#')
		mfp_printf("%s\r\n", ++line);
	else if (strlen(line) == 0)
		return;
	else
		run_command(line);
}

/*
 * Run script
 */
void run_script(int snum)
{
	int res;
	char *lp;
	char c;

	res = sd_load_script(snum, sbuf);

	if (res < 0)
	{
		mfp_printf("Could not load script %d\r\n", snum);
		return;
	}

	lp = &lbuf[0];

	for (int i = 0; i < res; i++)
	{
		c = sbuf[i];

		if (c == '\r')
			continue;
		else if (c == '\n')
		{
			*lp++ = '\0';
			run_line(lbuf);
			lp = &lbuf[0];
		}
		else
			*lp++ = c;
	}
}

/*
 * Check for init script (script 0) on SD card
 */
void run_init_script()
{
	if (!sd_chk_init_script())
		return;

	mfp_printf("\r\nInit script found, press enter to abort");

	mfp_set_uart_int(1);

	for (int i = 0; i < 10; i++)
	{
		if (mfp_getc_buf() == '\r')
		{
			mfp_printf("\r\n\r\n");
			mfp_set_uart_int(0);
			return;
		}

		mfp_delay_ms(500);
		mfp_printf(".");
	}

	mfp_set_uart_int(0);
	mfp_printf("\r\n\r\nRunning init script:\r\n\r\n");

	run_script(0);
}
