/*
 * T030 ROM BIOS - SPI bitbang routines for MFP GPIO
 *
 * (c) 2016 Tobias Rathje <tobbe@fasanix.com>
 *
 */

#include "mfp.h"
#include "util.h"

// GPIO lines on MFP
#define SPI_CLK 2
#define SPI_SI 3
#define SPI_SO 4
#define SPI_CS 5

void spi_init()
{
	// set output
	bit_set(MFPDDR, SPI_CLK);
	bit_set(MFPDDR, SPI_SI);
	bit_set(MFPDDR, SPI_CS);

	// set input
	bit_clear(MFPDDR, SPI_SO);

	// negate lines
	bit_clear(MFPGPDR, SPI_CLK);
	bit_clear(MFPGPDR, SPI_SI);
	bit_set(MFPGPDR, SPI_CS);
}

void spi_assert_cs()
{
	bit_clear(MFPGPDR, SPI_CS);
}

void spi_negate_cs()
{
	bit_set(MFPGPDR, SPI_CS);
}

void spi_write(uint8_t c)
{
	for (uint8_t mask = 0x80; mask > 0; mask >>= 1)
	{
		if (c & mask)
			bit_set(MFPGPDR, SPI_SI);
		else
			bit_clear(MFPGPDR, SPI_SI);

		// cycle clock
		bit_set(MFPGPDR, SPI_CLK);
		bit_clear(MFPGPDR, SPI_CLK);
	}
}

uint8_t spi_read()
{
	uint8_t c = 0;

	for (uint8_t i = 0; i < 8; i++)
	{
		if (bit_is_set(MFPGPDR, SPI_SO))
			c = c | 1;

		if (i < 7)
			c = c << 1;

		// cycle clock
		bit_set(MFPGPDR, SPI_CLK);
		bit_clear(MFPGPDR, SPI_CLK);
	}

	return c;
}
