/*
 * T030 ROM BIOS - TFTP implementation
 *
 * (c) 2016 Tobias Rathje <tobbe@fasanix.com>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ip_arp_udp_tcp.h"
#include "enc28j60.h"
#include "net.h"

#include "mfp.h"
#include "tftp.h"
#include "lxload.h"

#define TFTP_PORT 69
#define SRC_PORT 12345

#define BUFFER_SIZE 1000
static uint8_t buf[BUFFER_SIZE + 1];
static uint8_t tdata[600];

const char *oct_str = "octet";

uint16_t pr_cnt, prlf_cnt;
uint32_t w_addr, b_cnt;


void send_rrq(uint8_t *srvip, uint8_t *srvmac, char *fname)
{
	tdata[0] = 0x0;
	tdata[1] = 0x1; // RRQ

	uint8_t lc = 2;
	uint8_t *p;

	p = &tdata[2];

	for (uint8_t i = 0; i < strlen(fname); i++)
	{
		*p++ = *(fname + i);
		lc++;
	}

	*p++ = 0x0;
	lc++;

	for (uint8_t i = 0; i < strlen(oct_str); i++)
	{
		*p++ = *(oct_str + i);
		lc++;
	}

	*p++ = 0x0;
	lc++;

	send_udp(buf, (char *)tdata, lc, SRC_PORT, srvip, TFTP_PORT, srvmac);
}

int handle_packet(uint8_t *srvip, uint8_t *srvmac)
{
	uint16_t ulen, bl_num, srvport, dsize;
	uint8_t opcode;
	volatile unsigned char* mp;

	ulen = buf[UDP_LEN_H_P];
	ulen <<= 8;
	ulen |=  buf[UDP_LEN_L_P];

	srvport = buf[UDP_SRC_PORT_H_P];
	srvport <<= 8;
	srvport |= buf[UDP_SRC_PORT_L_P];

	opcode = buf[UDP_DATA_P + 1];

	if (opcode != 0x3)
	{
		if (opcode == 0x5)
		{
			char *err_msg = (char *)&buf[UDP_DATA_P + 4];
			mfp_printf("Error: %d - %s\r\n", buf[UDP_DATA_P + 3], err_msg);
			return -1;
		}

		mfp_printf("No data opcode: %d\r\n", opcode);
		return -2;
	}

	// data size
	dsize = ulen - 12;

	bl_num = buf[UDP_DATA_P + 2];
	bl_num <<= 8;
	bl_num |= buf[UDP_DATA_P + 3];

	pr_cnt++;

	if (pr_cnt > 10)
	{
		mfp_printf("#");
		pr_cnt = 0;

		prlf_cnt++;

		if (prlf_cnt > 70)
		{
			mfp_printf("\r\n");
			prlf_cnt = 0;
		}
	}

	// mfp_printf("\rReceived block: %d ", bl_num);

	// write block to memory
	for (int i = 0; i < dsize; i++)
	{
		mp = (unsigned char*)w_addr;
		*mp = buf[UDP_DATA_P + 4 + i];
		w_addr++;
	}

	b_cnt = b_cnt + dsize;

	tdata[0] = 0x0;
	tdata[1] = 0x4; // ACK
	tdata[2] = bl_num >> 8;
	tdata[3] = bl_num;

	// send ACK
	send_udp(buf, (char *)tdata, 4, SRC_PORT, srvip, srvport, srvmac);

	if (ulen - UDP_HEADER_LEN < 516)
	{
		mfp_printf("\r\n\r\nDone, received %d bytes\r\n", b_cnt);
		last_xfer_size = b_cnt;
		return 0;
	}
	else
		return 1;
}

void recv_file(uint8_t *srvip, uint8_t *srvmac, char *fname, uint32_t dst_addr)
{
	uint16_t plen, cnt;

	send_rrq(srvip, srvmac, fname);

	cnt = 30000;
	b_cnt = 0;
	pr_cnt = 0;
	prlf_cnt = 0;
	w_addr = dst_addr;

	while (1)
	{
		plen = enc28j60PacketReceive(BUFFER_SIZE, buf);

		if (plen > 0)
		{
			// src port 12345 = 0x3039
			if (buf[IP_PROTO_P] == IP_PROTO_UDP_V && buf[UDP_DST_PORT_H_P] == 0x30 && buf[UDP_DST_PORT_L_P] == 0x39)
			{
				int res = handle_packet(srvip, srvmac);

				if (res < 1)
					return;

				cnt = 0;
			}

			// handle ARP request
			if (eth_type_is_arp_and_my_ip(buf, plen))
			{
				if (buf[ETH_ARP_OPCODE_L_P] == ETH_ARP_OPCODE_REQ_L_V)
						make_arp_answer_from_request(buf);
			}
		}
		else
		{
			cnt--;

			if (cnt == 0)
			{
				mfp_printf("Timeout!\r\n");
				return;
			}
		}
	}
}
