/*
 * T030 ROM BIOS - Exception vector routines
 *
 * (c) 2016 Tobias Rathje <tobbe@fasanix.com>
 *
 */

#include <stdint.h>
#include <stdio.h>
#include "vector.h"
#include "mfp.h"

void set_vector(int vec, void (*pf)(void))
{
	uint32_t *vp = (uint32_t *)VEC_BASE;
	vp = vp + vec;
	*vp = (uint32_t)pf;
}

void init_vector()
{
	int i = 0;

	// install generic handler
	for (i = 0; i < 256; i++)
		set_vector(i, generic_handler);

	// set VBR
	void* vb=(void*)VEC_BASE;
	__asm("movec %0,%%vbr":: "d"(vb));
}

void __attribute__((optimize("O0"))) generic_handler()
{
	// remember that these should be static or they clog up the stack
	static uint32_t sp0, sp4, pc;
	static uint16_t sr, vo;
	static uint8_t vn;

	// get stack frame
	__asm("move.l %%sp@(0),%0":"=d"(sp0));
	__asm("move.l %%sp@(4),%0":"=d"(sp4));

	// status register
	sr = sp0 >> 16;

	// program counter
	pc = sp0 << 16;
	pc = pc | (sp4 >> 16);

	// vector offset
	vo = sp4 & 0x00000fff;

	// vector number
	vn = vo / 4;

	mfp_printf("Exception #%d ", vn);

	switch (vn)
	{
		case 2:
			mfp_printf("(bus error) ");
			break;
		case 3:
			mfp_printf("(address error) ");
			break;
		case 4:
			mfp_printf("(illegal instruction) ");
			break;
		case 5:
			mfp_printf("(division by zero) ");
			break;
		case 8:
			mfp_printf("(privilege violation) ");
			break;
		default:
			break;
	}

	mfp_printf("SR: %04x PC: %08x\r\n", sr, pc);

	// clear supervisor stack
	__asm("move.l #0x000ffffc, %sp");

	// crude jump to command interpreter loop
	__asm("jmp cmd_loop");
}
