/*
 * T030 ROM BIOS - XMODEM transfer
 *
 * (c) 2016 Tobias Rathje <tobbe@fasanix.com>
 *
 */

#include <stdio.h>
#include <string.h>
#include "mfp.h"
#include "xmodem.h"
#include "misc.h"

static uint32_t wr_addr;
static uint16_t err_count;

int write_mem(unsigned char* buffer, int size)
{
	unsigned char* bp;
	unsigned char* mp;
	bp = buffer;

	for (int i = 0; i < size; i++)
	{
		mp = (unsigned char*)wr_addr;
		*mp = *bp;

		if (*bp != *mp)
			err_count++;

		wr_addr++;
		bp++;
	}

	return 0;
}

void xfer_rcv(uint32_t addr)
{
	wr_addr = addr;
	err_count = 0;

    // xmodem init
    xmodemInit(&mfp_putc, &mfp_getc_buf);

    // enable interrupts
    sei();

	// enable interrupt for UART rx
    mfp_set_uart_int(1);

	mfp_printf("Starting XMODEM receive...\r\n");

	long res = xmodemReceive(&write_mem);

	xmodemInFlush();

	if (res < 0)
	{
		mfp_printf("\r\n\r\nReceive error: %ld", res);
	}
	else
	{
		mfp_printf("\r\n\r\nDone, bytes received: %ld, write errors: %d", res, err_count);
	}

	// disable interrupt
	mfp_set_uart_int(0);
}
